@@ -83,8 +83,7 @@ decode_attr_extra_data(struct tcb *const tcp,
 	data += attr_size;
 	size -= attr_size;
 
-	unsigned int i;
-	for (i = 0; i < size; ++i) {
+	for (unsigned int i = 0; i < size; ++i) {
 		if (data[i]) {
 			tprint_struct_next();
 			if (abbrev(tcp)) {
@@ -203,10 +202,7 @@ BEGIN_BPF_CMD_DECODER(BPF_MAP_CREATE)
 		 * without the access to the kernel headers.
 		 */
 		tprint_struct_next();
-		tprints_field_name("numa_node");
-		printxvals_ex(attr.numa_node, NULL,
-			      XLAT_STYLE_FMT_U | XLAT_STYLE_VERBOSE,
-			      numa_node, NULL);
+		PRINT_FIELD_XVAL_U_VERBOSE(attr, numa_node, numa_node, NULL);
 	}
 
 	/* map_name field was added in Linux commit v4.15-rc1~84^2~605^2~3. */
@@ -659,7 +655,7 @@ print_bpf_prog_info(struct tcb * const tcp, uint32_t bpf_fd,
 {
 	struct bpf_prog_info_struct info = { 0 };
 	const unsigned int len = MIN(size, bpf_prog_info_struct_size);
-	uint64_t map_id_buf;
+	uint32_t map_id_buf;
 
 	memcpy(&info, info_buf, len);
 
@@ -742,7 +738,7 @@ print_bpf_prog_info(struct tcb * const tcp, uint32_t bpf_fd,
 	print_big_u64_addr(info.map_ids);
 	print_array(tcp, info.map_ids, MIN(saved->nr_map_ids, info.nr_map_ids),
 		    &map_id_buf, sizeof(map_id_buf),
-		    tfetch_mem, print_uint32_array_member, 0);
+		    tfetch_mem, print_uint_array_member, 0);
 
 	tprint_struct_next();
 	PRINT_FIELD_CSTRING(info, name);
@@ -1041,7 +1037,7 @@ BEGIN_BPF_CMD_DECODER(BPF_PROG_QUERY)
 	print_big_u64_addr(attr.prog_ids);
 	print_array(tcp, attr.prog_ids, attr.prog_cnt, &prog_id_buf,
 		    sizeof(prog_id_buf), tfetch_mem,
-		    print_uint32_array_member, 0);
+		    print_uint_array_member, 0);
 
 	tprint_struct_next();
 	tprints_field_name("prog_cnt");
