@@ -322,6 +322,9 @@ struct tcb {
 	void *unwind_ctx;
 	struct unwind_queue_t *unwind_queue;
 # endif
+
+# define PROC_COMM_LEN 16
+	char comm[PROC_COMM_LEN];
 };
 
 /* TCB flags */
@@ -415,6 +418,7 @@ extern const struct xlat ip_type_of_services[];
 extern const struct xlat ipc_private[];
 extern const struct xlat msg_flags[];
 extern const struct xlat netlink_protocols[];
+extern const struct xlat nl_bridge_vlan_flags[];
 extern const struct xlat nl_netfilter_msg_types[];
 extern const struct xlat nl_route_types[];
 extern const struct xlat open_access_modes[];
@@ -422,6 +426,8 @@ extern const struct xlat open_mode_flags[];
 extern const struct xlat pollflags[];
 extern const struct xlat ptrace_cmds[];
 extern const struct xlat resource_flags[];
+extern const struct xlat route_nexthop_flags[];
+extern const struct xlat routing_protocols[];
 extern const struct xlat routing_scopes[];
 extern const struct xlat routing_table_ids[];
 extern const struct xlat routing_types[];
@@ -505,7 +511,7 @@ extern int Tflag_scale;
 extern int Tflag_width;
 extern bool iflag;
 extern bool count_wallclock;
-extern unsigned int pidns_translation;
+
 /* are we filtering traces based on paths? */
 extern struct path_set {
 	const char **paths_selected;
@@ -517,6 +523,12 @@ enum xflag_opts {
 	HEXSTR_NONE,
 	HEXSTR_NON_ASCII,
 	HEXSTR_ALL,
+	/*
+	 * It is here (after HEXSTR_ALL and not before HEXSTR_NON_ASCII) because
+	 * none/non-ascii/all are historic options values that are supported
+	 * via -x option.
+	 */
+	HEXSTR_NON_ASCII_CHARS,
 
 	NUM_HEXSTR_OPTS
 };
@@ -572,6 +584,23 @@ extern kernel_ulong_t get_rt_sigframe_addr(struct tcb *);
  *             is valid; NULL otherwise.
  */
 extern const char *syscall_name(kernel_ulong_t scno);
+/**
+ * Convert a pair of (raw) syscall number and arch (defined by an AUDIT_ARCH_*
+ * constant) to the corresponding syscall name.  So far works only for arches
+ * that are present in personalities supported by strace binary.
+ *
+ * @param nr     Raw syscall number.
+ * @param arch   AUDIT_ARCH_* constant that identifies the architecture.
+ * @param prefix If arch corresponds to current personality (and the argument
+ *               is non-NULL), an appropriate __NR_* constant prefix
+ *               is to be stored at the address pointed by the argument.
+ *               If arch corresponds to a different personality, or syscall
+ *               name has not been found, NULL is stored.
+ * @return       String literal corresponding to the syscall number in case
+ *               the latter is valid;  NULL otherwise.
+ */
+extern const char *syscall_name_arch(kernel_ulong_t nr, unsigned int arch,
+				     const char **prefix);
 /**
  * Convert a syscall name to the corresponding (shuffled) syscall number.
  *
@@ -588,9 +617,21 @@ extern kernel_long_t scno_by_name(const char *s, unsigned p,
  * The shuffling should be an involution: shuffle_scno(shuffle_scno(n)) == n.
  *
  * @param scno Raw or shuffled syscall number.
+ * @param pers Personality to shuffle scno for.
  * @return     Shuffled or raw syscall number, respectively.
  */
-extern kernel_ulong_t shuffle_scno(kernel_ulong_t scno);
+extern kernel_ulong_t shuffle_scno_pers(kernel_ulong_t scno, int pers);
+/**
+ * Shorthand for shuffle_scno_pers() for the current personality.
+ *
+ * @param scno Raw or shuffled syscall number.
+ * @return     Shuffled or raw syscall number, respectively.
+ */
+static inline kernel_ulong_t
+shuffle_scno(kernel_ulong_t scno)
+{
+	return shuffle_scno_pers(scno, current_personality);
+}
 /**
  * Print error name in accordance with current xlat style setting.
  *
@@ -680,6 +721,24 @@ umoven_or_printaddr(struct tcb *tcp, const kernel_ulong_t addr,
 # define umove_or_printaddr(pid, addr, objp)	\
 	umoven_or_printaddr((pid), (addr), sizeof(*(objp)), (void *) (objp))
 
+/**
+ * Retrieves up to 8 bytes (that are treated as a single unsigned integer value)
+ * and puts it into variable pointed by laddr;
+ *
+ * @return 0 on success, -1 (and print addr) on error or if len is more
+ *         than size of uint64_t.
+ */
+extern int
+umoven_to_uint64_or_printaddr64(struct tcb *, uint64_t addr,
+				unsigned int len, uint64_t *laddr);
+
+static inline int
+umoven_to_uint64_or_printaddr(struct tcb *tcp, const kernel_ulong_t addr,
+			      unsigned int len, uint64_t *laddr)
+{
+	return umoven_to_uint64_or_printaddr64(tcp, addr, len, laddr);
+}
+
 /**
  * @return 0 on success, -1 on error (and print addr).
  */
@@ -925,44 +984,15 @@ extern void print_kernel_version(unsigned long version);
 extern void print_abnormal_hi(kernel_ulong_t);
 extern void print_ioprio(unsigned int ioprio);
 
-extern bool print_int8_array_member(struct tcb *, void *elem_buf,
+extern bool print_int_array_member(struct tcb *, void *elem_buf,
+				   size_t elem_size, void *data);
+extern bool print_uint_array_member(struct tcb *, void *elem_buf,
+				    size_t elem_size, void *data);
+extern bool print_xint_array_member(struct tcb *, void *elem_buf,
 				    size_t elem_size, void *data);
-extern bool print_uint8_array_member(struct tcb *, void *elem_buf,
-				     size_t elem_size, void *data);
-extern bool print_xint8_array_member(struct tcb *, void *elem_buf,
-				      size_t elem_size, void *data);
-extern bool print_int32_array_member(struct tcb *, void *elem_buf,
-				     size_t elem_size, void *data);
-extern bool print_uint32_array_member(struct tcb *, void *elem_buf,
-				      size_t elem_size, void *data);
-extern bool print_xint32_array_member(struct tcb *, void *elem_buf,
-				      size_t elem_size, void *data);
-extern bool print_uint64_array_member(struct tcb *, void *elem_buf,
-				      size_t elem_size, void *data);
-extern bool print_xint64_array_member(struct tcb *, void *elem_buf,
-				      size_t elem_size, void *data);
 extern bool print_fd_array_member(struct tcb *, void *elem_buf,
 				  size_t elem_size, void *data);
 
-static inline bool
-print_xlong_array_member(struct tcb *tcp, void *elem_buf, size_t elem_size,
-			 void *data)
-{
-	return dispatch_wordsize(print_xint64_array_member,
-				 print_xint32_array_member,
-				 tcp, elem_buf, elem_size, data);
-}
-
-static inline bool
-print_kulong_array_member(struct tcb *tcp, void *elem_buf, size_t elem_size,
-			  void *data)
-{
-	return dispatch_klongsize(print_uint64_array_member,
-				  print_uint32_array_member,
-				  tcp, elem_buf, elem_size, data);
-}
-
-
 typedef bool (*tfetch_mem_fn)(struct tcb *, kernel_ulong_t addr,
 			      unsigned int size, void *dest);
 typedef bool (*print_fn)(struct tcb *, void *elem_buf,
@@ -976,29 +1006,32 @@ typedef void (*print_obj_by_addr_size_fn)(struct tcb *,
 
 
 /**
- * Array printing function with over-engineered interface.
+ * Array printing function with an over-engineered interface.
  *
- * @param start_addr       If tfetch_mem_fn is non-NULL: address in tracee's
+ * @param start_addr       If tfetch_mem_fn is non-NULL: an address in tracee's
  *                         memory where the start of the array is located.
  *                         If tfetch_mem_fn is NULL: ignored.
- * @param nmemb            Number of elements in array.
+ * @param nmemb            Number of elements in the array.
  * @param elem_buf         If tfetch_mem_fn is non-NULL: a buffer where each
  *                         element fetched by tfetch_mem_fn is stored.
- *                         If tfetch_mem_fn is NULL: address of the start of
- *                         the array in local memory.
- * @param elem_size        Size (in bytes) of each element in the array.
- * @param tfetch_mem_fn    Fetching function. If NULL, then elem_buf is treated
- *                         as local array of nmemb members elem_size each;
- *                         start_addr is ignored.
+ *                         If tfetch_mem_fn is NULL: an address of the start
+ *                         of the array in strace's memory.
+ * @param elem_size        Size (in bytes) of the each element in the array.
+ * @param tfetch_mem_fn    Fetching function.  If it is NULL, then elem_buf
+ *                         is treated as a local array of nmemb members
+ *                         of elem_size each, and start_addr is ignored.
  * @param print_func       Element printing callback.
  * @param opaque_data      A value that is unconditionally passed to print_func
- *                         in opaque_data argument.
+ *                         in the opaque_data argument.
  * @param flags            Combination of xlat style settings and additional
- *                         flags from enum print_array_flags.
- * @param index_xlat       Xlat array that is used for printing indices.
- * @param index_xlat_size  The size of xlat array.
+ *                         flags from enum print_array_flags that are appled
+ *                         to index printing.
+ * @param index_xlat       xlat that is used for printing indices.
  * @param index_dflt       Default string for the values not found
  *                         in index_xlat.
+ * @return                 true if at least one element has been fetched
+ *                         successfully (tfetch_mem_fn returned true);
+ *                         false otherwise.
  */
 extern bool
 print_array_ex(struct tcb *,
@@ -1205,7 +1238,6 @@ decode_inet_addr(struct tcb *, kernel_ulong_t addr,
 extern void print_ax25_addr(const void /* ax25_address */ *addr);
 extern void print_x25_addr(const void /* struct x25_address */ *addr);
 extern const char *get_sockaddr_by_inode(struct tcb *, int fd, unsigned long inode);
-extern bool print_sockaddr_by_inode(struct tcb *, int fd, unsigned long inode);
 
 /**
  * Prints dirfd file descriptor and saves it in tcp->last_dirfd,
@@ -1277,6 +1309,18 @@ extern void print_ifindex(unsigned int);
 
 extern void print_bpf_filter_code(const uint16_t code, bool extended);
 
+/**
+ * Prints a CPU set, pointed at by addr, as a list of CPU IDs.  CPU set
+ * (CPU mask, CPU affinity set, CPU affinity list) is a bit set represented
+ * as an array of elements of unsigned long type.
+ *
+ * @param tcp  TCB pointer.
+ * @param addr An address of the CPU set.
+ * @param len  Length of CPU set in bytes.
+ */
+extern void print_affinitylist(struct tcb *const tcp, const kernel_ulong_t addr,
+			       const unsigned int len);
+
 extern void qualify(const char *);
 extern void qualify_trace(const char *);
 extern void qualify_abbrev(const char *);
@@ -1286,6 +1330,7 @@ extern void qualify_signals(const char *);
 extern void qualify_status(const char *);
 extern void qualify_quiet(const char *);
 extern void qualify_decode_fd(const char *);
+extern void qualify_decode_pid(const char *);
 extern void qualify_read(const char *);
 extern void qualify_write(const char *);
 extern void qualify_fault(const char *);
@@ -1305,11 +1350,13 @@ DECL_IOCTL(fs_f);
 DECL_IOCTL(fs_x);
 DECL_IOCTL(gpio);
 DECL_IOCTL(inotify);
+DECL_IOCTL(kd);
 DECL_IOCTL(kvm);
 DECL_IOCTL(nbd);
 DECL_IOCTL(nsfs);
 DECL_IOCTL(ptp);
 DECL_IOCTL(random);
+DECL_IOCTL(seccomp);
 DECL_IOCTL(scsi);
 DECL_IOCTL(tee);
 DECL_IOCTL(term);
@@ -1423,6 +1470,9 @@ extern const struct timespec *ts_min(const struct timespec *, const struct times
 extern const struct timespec *ts_max(const struct timespec *, const struct timespec *);
 extern int parse_ts(const char *s, struct timespec *t);
 
+/** Print a clock_t value, as used by times(2) (and not clock(3)). */
+extern void print_clock_t(uint64_t val);
+
 # ifdef ENABLE_STACKTRACE
 extern void unwind_init(void);
 extern void unwind_tcb_init(struct tcb *);
@@ -1436,6 +1486,10 @@ extern void kvm_run_structure_decoder_init(void);
 extern void kvm_vcpu_info_free(struct tcb *);
 # endif
 
+extern void maybe_load_task_comm(struct tcb *tcp);
+/* Print the contents of /proc/$pid/comm. */
+extern void print_pid_comm(int pid);
+
 static inline int
 printstrn(struct tcb *tcp, kernel_ulong_t addr, kernel_ulong_t len)
 {
@@ -1707,6 +1761,32 @@ truncate_kulong_to_current_wordsize(const kernel_ulong_t v)
 	}
 }
 
+static inline kernel_long_t
+truncate_klong_to_current_klongsize(const kernel_long_t v)
+{
+# if ANY_WORDSIZE_LESS_THAN_KERNEL_LONG
+	if (current_klongsize < sizeof(v)) {
+		return (int) v;
+	} else
+# endif
+	{
+		return v;
+	}
+}
+
+static inline kernel_ulong_t
+truncate_kulong_to_current_klongsize(const kernel_ulong_t v)
+{
+# if ANY_WORDSIZE_LESS_THAN_KERNEL_LONG
+	if (current_klongsize < sizeof(v)) {
+		return (unsigned int) v;
+	} else
+# endif
+	{
+		return v;
+	}
+}
+
 /*
  * Cast a pointer or a pointer-sized integer to kernel_ulong_t.
  */
@@ -1781,6 +1861,13 @@ extern const unsigned int nsyscall_vec[SUPPORTED_PERSONALITIES];
 extern const struct_sysent *const sysent_vec[SUPPORTED_PERSONALITIES];
 extern struct inject_opts *inject_vec[SUPPORTED_PERSONALITIES];
 
+struct audit_arch_t {
+	unsigned int arch; /* AUDIT_ARCH_* */
+	unsigned int flag;
+};
+
+extern const struct audit_arch_t audit_arch_vec[SUPPORTED_PERSONALITIES];
+
 # ifdef ENABLE_COVERAGE_GCOV
 #  ifdef HAVE_GCOV_H
 #   include <gcov.h>
