@@ -21,6 +21,7 @@ struct number_set *signal_set;
 struct number_set *status_set;
 struct number_set *quiet_set;
 struct number_set *decode_fd_set;
+struct number_set *decode_pid_set;
 struct number_set *trace_set;
 
 bool quiet_set_updated = false;
@@ -108,6 +109,17 @@ decode_fd_str_to_uint(const char *str)
 	return (int) find_arg_val(str, decode_fd_strs, -1ULL, -1ULL);
 }
 
+static int
+decode_pid_str_to_uint(const char *str)
+{
+	static const struct xlat_data decode_pid_strs[] = {
+		{ DECODE_PID_NS_TRANSLATION,	"pidns" },
+		{ DECODE_PID_COMM,		"comm" },
+	};
+
+	return (int) find_arg_val(str, decode_pid_strs, -1ULL, -1ULL);
+}
+
 static int
 find_errno_by_name(const char *name)
 {
@@ -143,11 +155,7 @@ parse_delay_token(const char *input, struct inject_opts *fopts, bool isenter)
 static bool
 parse_poke_token(const char *input, struct inject_opts *fopts, bool isenter)
 {
-	char *token, *str_tokenized, *saveptr = NULL;
-	const char *val;
-	size_t data_len;
 	unsigned flag = isenter ? INJECT_F_POKE_ENTER : INJECT_F_POKE_EXIT;
-	struct poke_payload *poke;
 
 	/* disallow duplicates */
 	if (fopts->data.flags & flag)
@@ -160,15 +168,17 @@ parse_poke_token(const char *input, struct inject_opts *fopts, bool isenter)
 	if (fopts->data.poke_idx == (uint16_t) -1)
 		fopts->data.poke_idx = alloc_poke_data();
 
-	saveptr = NULL;
-	str_tokenized = xstrdup(input);
-	for (token = strtok_r(str_tokenized, ",", &saveptr);
+	char *saveptr = NULL;
+	char *str_tokenized = xstrdup(input);
+	struct poke_payload *poke;
+	for (char *token = strtok_r(str_tokenized, ",", &saveptr);
 	     token;
 	     token = strtok_r(NULL, ",", &saveptr)) {
 		poke = xcalloc(1, sizeof(*poke));
 		poke->is_enter = isenter;
 
-		if ((val = STR_STRIP_PREFIX(token, "@arg")) == token)
+		const char *val = STR_STRIP_PREFIX(token, "@arg");
+		if (val == token)
 			goto err;
 		if ((val[0] >= '1') && (val[0] <= '7')) {
 			poke->arg_no = val[0] - '0';
@@ -179,7 +189,7 @@ parse_poke_token(const char *input, struct inject_opts *fopts, bool isenter)
 			goto err;
 		val += 2;
 
-		data_len = strlen(val);
+		size_t data_len = strlen(val);
 		if ((data_len == 0) || (data_len % 2) || (data_len > 2048))
 			goto err;
 		data_len /= 2;
@@ -205,7 +215,7 @@ err:
 	return false;
 }
 
-static inline void
+static void
 check_inadvertent_fault_injection(unsigned long long ullval)
 {
 #if ANY_WORDSIZE_LESS_THAN_KERNEL_LONG
@@ -467,6 +477,15 @@ qualify_decode_fd(const char *const str)
 		       "decode-fds");
 }
 
+void
+qualify_decode_pid(const char *const str)
+{
+	if (!decode_pid_set)
+		decode_pid_set = alloc_number_set_array(1);
+	qualify_tokens(str, decode_pid_set, decode_pid_str_to_uint,
+		       "decode-pids");
+}
+
 void
 qualify_trace(const char *const str)
 {
@@ -632,6 +651,8 @@ static const struct qual_options {
 	{ "kvm",	qualify_kvm	},
 	{ "decode-fd",	qualify_decode_fd },
 	{ "decode-fds",	qualify_decode_fd },
+	{ "decode-pid",	qualify_decode_pid },
+	{ "decode-pids", qualify_decode_pid },
 };
 
 void
