@@ -13,6 +13,7 @@
 # include <sys/types.h>
 
 # include "gcc_compat.h"
+# include "static_assert.h"
 
 # define ARRAY_SIZE(a_)	(sizeof(a_) / sizeof((a_)[0]) + MUST_BE_ARRAY(a_))
 
@@ -108,4 +109,24 @@ is_filled(const char *ptr, char fill, size_t size)
 
 # define FLAG(name_) name_ = BIT32(name_##_BIT)
 
+/**
+ * A shorthand for a build-time check of a type size that provides
+ * a corresponding "update the decoder" message in a case of failure.
+ * @param type_ Type whose size is to be checked.
+ * @param sz_   Expected type size in bytes.
+ */
+# define CHECK_TYPE_SIZE(type_, sz_) \
+	static_assert(sizeof(type_) == (sz_), \
+		      "Unexpected size of " #type_ "(" #sz_ " expected)")
+
+# ifdef WORDS_BIGENDIAN
+#  define BE16(val_) val_
+#  define BE32(val_) val_
+#  define BE64(val_) val_
+# else
+#  define BE16(val_) ((((val_) & 0xff) << 8) | (((val_) >> 8) & 0xff))
+#  define BE32(val_) ((BE16(val_) << 16) | BE16((val_) >> 16))
+#  define BE64(val_) ((BE32(val_) << 32) | BE32((val_) >> 32))
+# endif
+
 #endif /* !STRACE_MACROS_H */
