@@ -28,15 +28,8 @@
 #include <arpa/inet.h>
 #include <net/if.h>
 #include <asm/types.h>
-#ifdef HAVE_NETIPX_IPX_H
-# include <netipx/ipx.h>
-#else
-# include <linux/ipx.h>
-#endif
 
-#if defined(HAVE_LINUX_IP_VS_H)
-# include <linux/ip_vs.h>
-#endif
+#include <linux/ip_vs.h>
 #include "netlink.h"
 #if defined(HAVE_LINUX_NETFILTER_ARP_ARP_TABLES_H)
 # include <linux/netfilter_arp/arp_tables.h>
@@ -52,6 +45,7 @@
 #endif
 #include <linux/if_packet.h>
 #include <linux/icmp.h>
+#include <linux/vm_sockets.h>
 
 #include "xlat/socktypes.h"
 #include "xlat/sock_type_flags.h"
@@ -78,8 +72,6 @@
 #include "xlat/kcm_protocols.h"
 #include "xlat/smc_protocols.h"
 
-const size_t inet_protocols_size = ARRAY_SIZE(inet_protocols) - 1;
-
 static void
 decode_sockbuf(struct tcb *const tcp, const int fd, const kernel_ulong_t addr,
 	       const kernel_ulong_t addrlen)
@@ -115,10 +107,15 @@ tprint_sock_type(unsigned int flags)
 
 SYS_FUNC(socket)
 {
+	/* domain */
 	printxval(addrfams, tcp->u_arg[0], "AF_???");
-	tprints(", ");
+	tprint_arg_next();
+
+	/* type */
 	tprint_sock_type(tcp->u_arg[1]);
-	tprints(", ");
+	tprint_arg_next();
+
+	/* protocol */
 	switch (tcp->u_arg[0]) {
 	case AF_INET:
 	case AF_INET6:
@@ -136,10 +133,10 @@ SYS_FUNC(socket)
 		break;
 
 	case AF_PACKET:
-		tprints("htons(");
+		tprints_arg_begin("htons");
 		printxval(ethernet_protocols, ntohs(tcp->u_arg[2]),
 			  "ETH_P_???");
-		tprints(")");
+		tprint_arg_end();
 		break;
 
 	case AF_IRDA:
@@ -183,7 +180,7 @@ SYS_FUNC(socket)
 		break;
 
 	default:
-		tprintf("%" PRI_klu, tcp->u_arg[2]);
+		PRINT_VAL_U(tcp->u_arg[2]);
 		break;
 	}
 
@@ -204,15 +201,21 @@ decode_sockname(struct tcb *tcp)
 	int ulen, rlen;
 
 	if (entering(tcp)) {
+		/* sockfd */
 		printfd(tcp, tcp->u_arg[0]);
-		tprints(", ");
+		tprint_arg_next();
+
 		if (fetch_socklen(tcp, &ulen, tcp->u_arg[1], tcp->u_arg[2])) {
 			set_tcb_priv_ulong(tcp, ulen);
 			return 0;
 		} else {
+			/* addr */
 			printaddr(tcp->u_arg[1]);
-			tprints(", ");
+			tprint_arg_next();
+
+			/* addrlen */
 			printaddr(tcp->u_arg[2]);
+
 			return RVAL_DECODED;
 		}
 	}
@@ -220,14 +223,27 @@ decode_sockname(struct tcb *tcp)
 	ulen = get_tcb_priv_ulong(tcp);
 
 	if (syserror(tcp) || umove(tcp, tcp->u_arg[2], &rlen) < 0) {
+		/* addr */
 		printaddr(tcp->u_arg[1]);
-		tprintf(", [%d]", ulen);
+		tprint_arg_next();
+
+		/* addrlen */
+		tprint_indirect_begin();
+		PRINT_VAL_D(ulen);
+		tprint_indirect_end();
 	} else {
+		/* addr */
 		decode_sockaddr(tcp, tcp->u_arg[1], ulen > rlen ? rlen : ulen);
-		if (ulen != rlen)
-			tprintf(", [%d->%d]", ulen, rlen);
-		else
-			tprintf(", [%d]", rlen);
+		tprint_arg_next();
+
+		/* addrlen */
+		tprint_indirect_begin();
+		if (ulen != rlen) {
+			PRINT_VAL_D(ulen);
+			tprint_value_changed();
+		}
+		PRINT_VAL_D(rlen);
+		tprint_indirect_end();
 	}
 
 	return RVAL_DECODED;
@@ -243,7 +259,8 @@ SYS_FUNC(accept4)
 	int rc = decode_sockname(tcp);
 
 	if (rc & RVAL_DECODED) {
-		tprints(", ");
+		/* flags */
+		tprint_arg_next();
 		printflags(sock_type_flags, tcp->u_arg[3], "SOCK_???");
 	}
 
@@ -252,10 +269,18 @@ SYS_FUNC(accept4)
 
 SYS_FUNC(send)
 {
+	/* sockfd */
 	printfd(tcp, tcp->u_arg[0]);
-	tprints(", ");
+	tprint_arg_next();
+
+	/* buf */
 	decode_sockbuf(tcp, tcp->u_arg[0], tcp->u_arg[1], tcp->u_arg[2]);
-	tprintf(", %" PRI_klu ", ", tcp->u_arg[2]);
+	tprint_arg_next();
+
+	/* len */
+	PRINT_VAL_U(tcp->u_arg[2]);
+	tprint_arg_next();
+
 	/* flags */
 	printflags(msg_flags, tcp->u_arg[3], "MSG_???");
 
@@ -264,18 +289,29 @@ SYS_FUNC(send)
 
 SYS_FUNC(sendto)
 {
+	/* sockfd */
 	printfd(tcp, tcp->u_arg[0]);
-	tprints(", ");
+	tprint_arg_next();
+
+	/* buf */
 	decode_sockbuf(tcp, tcp->u_arg[0], tcp->u_arg[1], tcp->u_arg[2]);
-	tprintf(", %" PRI_klu ", ", tcp->u_arg[2]);
+	tprint_arg_next();
+
+	/* len */
+	PRINT_VAL_U(tcp->u_arg[2]);
+	tprint_arg_next();
+
 	/* flags */
 	printflags(msg_flags, tcp->u_arg[3], "MSG_???");
-	/* to address */
+	tprint_arg_next();
+
+	/* dest_addr */
 	const int addrlen = tcp->u_arg[5];
-	tprints(", ");
 	decode_sockaddr(tcp, tcp->u_arg[4], addrlen);
-	/* to length */
-	tprintf(", %d", addrlen);
+	tprint_arg_next();
+
+	/* addrlen */
+	PRINT_VAL_D(addrlen);
 
 	return RVAL_DECODED;
 }
@@ -283,9 +319,11 @@ SYS_FUNC(sendto)
 SYS_FUNC(recv)
 {
 	if (entering(tcp)) {
+		/* sockfd */
 		printfd(tcp, tcp->u_arg[0]);
-		tprints(", ");
+		tprint_arg_next();
 	} else {
+		/* buf */
 		if (syserror(tcp)) {
 			printaddr(tcp->u_arg[1]);
 		} else {
@@ -293,8 +331,13 @@ SYS_FUNC(recv)
 				       MIN((kernel_ulong_t) tcp->u_rval,
 					   tcp->u_arg[2]));
 		}
+		tprint_arg_next();
 
-		tprintf(", %" PRI_klu ", ", tcp->u_arg[2]);
+		/* len */
+		PRINT_VAL_U(tcp->u_arg[2]);
+		tprint_arg_next();
+
+		/* flags */
 		printflags(msg_flags, tcp->u_arg[3], "MSG_???");
 	}
 	return 0;
@@ -305,8 +348,10 @@ SYS_FUNC(recvfrom)
 	int ulen, rlen;
 
 	if (entering(tcp)) {
+		/* sockfd */
 		printfd(tcp, tcp->u_arg[0]);
-		tprints(", ");
+		tprint_arg_next();
+
 		if (fetch_socklen(tcp, &ulen, tcp->u_arg[4], tcp->u_arg[5])) {
 			set_tcb_priv_ulong(tcp, ulen);
 		}
@@ -319,36 +364,53 @@ SYS_FUNC(recvfrom)
 				       MIN((kernel_ulong_t) tcp->u_rval,
 					   tcp->u_arg[2]));
 		}
-		/* size */
-		tprintf(", %" PRI_klu ", ", tcp->u_arg[2]);
+		tprint_arg_next();
+
+		/* len */
+		PRINT_VAL_U(tcp->u_arg[2]);
+		tprint_arg_next();
+
 		/* flags */
 		printflags(msg_flags, tcp->u_arg[3], "MSG_???");
-		tprints(", ");
+		tprint_arg_next();
 
 		ulen = get_tcb_priv_ulong(tcp);
 
 		if (!fetch_socklen(tcp, &rlen, tcp->u_arg[4], tcp->u_arg[5])) {
-			/* from address */
+			/* src_addr */
 			printaddr(tcp->u_arg[4]);
-			tprints(", ");
-			/* from length */
+			tprint_arg_next();
+
+			/* addrlen */
 			printaddr(tcp->u_arg[5]);
+
 			return 0;
 		}
 		if (syserror(tcp)) {
-			/* from address */
+			/* src_addr */
 			printaddr(tcp->u_arg[4]);
-			/* from length */
-			tprintf(", [%d]", ulen);
+			tprint_arg_next();
+
+			/* addrlen */
+			tprint_indirect_begin();
+			PRINT_VAL_D(ulen);
+			tprint_indirect_end();
+
 			return 0;
 		}
-		/* from address */
+
+		/* src_addr */
 		decode_sockaddr(tcp, tcp->u_arg[4], ulen > rlen ? rlen : ulen);
-		/* from length */
-		if (ulen != rlen)
-			tprintf(", [%d->%d]", ulen, rlen);
-		else
-			tprintf(", [%d]", rlen);
+		tprint_arg_next();
+
+		/* addrlen */
+		tprint_indirect_begin();
+		if (ulen != rlen) {
+			PRINT_VAL_D(ulen);
+			tprint_value_changed();
+		}
+		PRINT_VAL_D(rlen);
+		tprint_indirect_end();
 	}
 	return 0;
 }
@@ -358,34 +420,23 @@ SYS_FUNC(getsockname)
 	return decode_sockname(tcp);
 }
 
-static void
-printpair_fd(struct tcb *tcp, const int i0, const int i1)
-{
-	tprints("[");
-	printfd(tcp, i0);
-	tprints(", ");
-	printfd(tcp, i1);
-	tprints("]");
-}
-
 static void
 decode_pair_fd(struct tcb *const tcp, const kernel_ulong_t addr)
 {
-	int pair[2];
-
-	if (umove_or_printaddr(tcp, addr, &pair))
-		return;
-
-	printpair_fd(tcp, pair[0], pair[1]);
+	int fd;
+	print_array(tcp, addr, 2, &fd, sizeof(fd),
+		    tfetch_mem, print_fd_array_member, NULL);
 }
 
 static int
 do_pipe(struct tcb *tcp, int flags_arg)
 {
 	if (exiting(tcp)) {
+		/* pipefd */
 		decode_pair_fd(tcp, tcp->u_arg[0]);
 		if (flags_arg >= 0) {
-			tprints(", ");
+			/* flags */
+			tprint_arg_next();
 			printflags(open_mode_flags, tcp->u_arg[flags_arg], "O_???");
 		}
 	}
@@ -395,8 +446,13 @@ do_pipe(struct tcb *tcp, int flags_arg)
 SYS_FUNC(pipe)
 {
 #if HAVE_ARCH_GETRVAL2
-	if (exiting(tcp) && !syserror(tcp))
-		printpair_fd(tcp, tcp->u_rval, getrval2(tcp));
+	if (exiting(tcp) && !syserror(tcp)) {
+		tprint_array_begin();
+		printfd(tcp, tcp->u_rval);
+		tprint_array_next();
+		printfd(tcp, getrval2(tcp));
+		tprint_array_end();
+	}
 	return 0;
 #else
 	return do_pipe(tcp, -1);
@@ -411,12 +467,19 @@ SYS_FUNC(pipe2)
 SYS_FUNC(socketpair)
 {
 	if (entering(tcp)) {
+		/* domain */
 		printxval(addrfams, tcp->u_arg[0], "AF_???");
-		tprints(", ");
+		tprint_arg_next();
+
+		/* type */
 		tprint_sock_type(tcp->u_arg[1]);
-		tprintf(", %" PRI_klu, tcp->u_arg[2]);
+		tprint_arg_next();
+
+		/* protocol */
+		PRINT_VAL_U(tcp->u_arg[2]);
+		tprint_arg_next();
 	} else {
-		tprints(", ");
+		/* sv */
 		decode_pair_fd(tcp, tcp->u_arg[3]);
 	}
 	return 0;
@@ -428,6 +491,7 @@ SYS_FUNC(socketpair)
 #include "xlat/sock_ip_options.h"
 #include "xlat/getsock_ip_options.h"
 #include "xlat/setsock_ip_options.h"
+#include "xlat/sock_vsock_options.h"
 #include "xlat/sock_ipv6_options.h"
 #include "xlat/getsock_ipv6_options.h"
 #include "xlat/setsock_ipv6_options.h"
@@ -439,6 +503,7 @@ SYS_FUNC(socketpair)
 #include "xlat/sock_sctp_options.h"
 #include "xlat/sock_tcp_options.h"
 #include "xlat/sock_udp_options.h"
+#include "xlat/sock_can_raw_options.h"
 #include "xlat/sock_irda_options.h"
 #include "xlat/sock_llc_options.h"
 #include "xlat/sock_dccp_options.h"
@@ -456,7 +521,7 @@ SYS_FUNC(socketpair)
 #include "xlat/sock_tls_options.h"
 #include "xlat/sock_xdp_options.h"
 
-#define MAYBE_PRINT_FIELD_LEN(print_prefix_, where_, field_, 		\
+#define MAYBE_PRINT_FIELD_LEN(print_prefix_, where_, field_,		\
 			len_, print_func_, ...)				\
 	do {								\
 		unsigned int start = offsetof(typeof(where_), field_);	\
@@ -479,11 +544,15 @@ static void
 print_sockopt_fd_level_name(struct tcb *tcp, int fd, unsigned int level,
 			    unsigned int name, bool is_getsockopt)
 {
+	/* sockfd */
 	printfd(tcp, fd);
-	tprints(", ");
-	printxval(socketlayers, level, "SOL_??");
-	tprints(", ");
+	tprint_arg_next();
 
+	/* level */
+	printxval(socketlayers, level, "SOL_??");
+	tprint_arg_next();
+
+	/* optname */
 	switch (level) {
 	case SOL_SOCKET:
 		printxvals(name, "SO_???", sock_options,
@@ -495,6 +564,14 @@ print_sockopt_fd_level_name(struct tcb *tcp, int fd, unsigned int level,
 			   is_getsockopt ? getsock_ip_options :
 					   setsock_ip_options, NULL);
 		break;
+	/*
+	 * Yes, VMWare in their infinite wisdom have decided to use address
+	 * family instead of a socket option layer for the socket option layer
+	 * check, see net/vmw_vsock/af_vsock.c:vsock_connectible_[gs]etsockopt.
+	 */
+	case AF_VSOCK:
+		printxval(sock_vsock_options, name, "SO_VM_???");
+		break;
 	case SOL_IPV6:
 		printxvals(name, "IPV6_???", sock_ipv6_options,
 			   is_getsockopt ? getsock_ipv6_options :
@@ -512,6 +589,9 @@ print_sockopt_fd_level_name(struct tcb *tcp, int fd, unsigned int level,
 	case SOL_TCP:
 		printxval(sock_tcp_options, name, "TCP_???");
 		break;
+	case SOL_CAN_RAW:
+		printxval(sock_can_raw_options, name, "CAN_RAW_???");
+		break;
 	case SOL_SCTP:
 		printxval(sock_sctp_options, name, "SCTP_???");
 		break;
@@ -576,10 +656,8 @@ print_sockopt_fd_level_name(struct tcb *tcp, int fd, unsigned int level,
 		/* Other SOL_* protocol levels still need work. */
 
 	default:
-		tprintf("%u", name);
+		PRINT_VAL_U(name);
 	}
-
-	tprints(", ");
 }
 
 static void
@@ -642,9 +720,9 @@ print_get_error(struct tcb *const tcp, const kernel_ulong_t addr,
 	if (umoven_or_printaddr(tcp, addr, len, &err))
 		return;
 
-	tprints("[");
+	tprint_indirect_begin();
 	print_err(err, false);
-	tprints("]");
+	tprint_indirect_end();
 }
 
 #ifdef PACKET_STATISTICS
@@ -766,7 +844,7 @@ print_getsockopt(struct tcb *const tcp, const unsigned int level,
 			uint32_t buf;
 			print_array(tcp, addr, MIN(ulen, rlen) / sizeof(buf),
 				    &buf, sizeof(buf),
-				    tfetch_mem, print_uint32_array_member, 0);
+				    tfetch_mem, print_uint_array_member, 0);
 			break;
 			}
 		default:
@@ -796,14 +874,18 @@ SYS_FUNC(getsockopt)
 	if (entering(tcp)) {
 		print_sockopt_fd_level_name(tcp, tcp->u_arg[0],
 					    tcp->u_arg[1], tcp->u_arg[2], true);
+		tprint_arg_next();
 
 		if (verbose(tcp) && tcp->u_arg[4]
 		    && umove(tcp, tcp->u_arg[4], &ulen) == 0) {
 			set_tcb_priv_ulong(tcp, ulen);
 			return 0;
 		} else {
+			/* optval */
 			printaddr(tcp->u_arg[3]);
-			tprints(", ");
+			tprint_arg_next();
+
+			/* optlen */
 			printaddr(tcp->u_arg[4]);
 			return RVAL_DECODED;
 		}
@@ -811,15 +893,28 @@ SYS_FUNC(getsockopt)
 		ulen = get_tcb_priv_ulong(tcp);
 
 		if (syserror(tcp) || umove(tcp, tcp->u_arg[4], &rlen) < 0) {
+			/* optval */
 			printaddr(tcp->u_arg[3]);
-			tprintf(", [%d]", ulen);
+			tprint_arg_next();
+
+			/* optlen */
+			tprint_indirect_begin();
+			PRINT_VAL_D(ulen);
+			tprint_indirect_end();
 		} else {
+			/* optval */
 			print_getsockopt(tcp, tcp->u_arg[1], tcp->u_arg[2],
 					 tcp->u_arg[3], ulen, rlen);
-			if (ulen != rlen)
-				tprintf(", [%d->%d]", ulen, rlen);
-			else
-				tprintf(", [%d]", rlen);
+			tprint_arg_next();
+
+			/* optlen */
+			tprint_indirect_begin();
+			if (ulen != rlen) {
+				PRINT_VAL_D(ulen);
+				tprint_value_changed();
+			}
+			PRINT_VAL_D(rlen);
+			tprint_indirect_end();
 		}
 	}
 	return 0;
@@ -1043,9 +1138,15 @@ SYS_FUNC(setsockopt)
 {
 	print_sockopt_fd_level_name(tcp, tcp->u_arg[0],
 				    tcp->u_arg[1], tcp->u_arg[2], false);
+	tprint_arg_next();
+
+	/* optval */
 	print_setsockopt(tcp, tcp->u_arg[1], tcp->u_arg[2],
 			 tcp->u_arg[3], tcp->u_arg[4]);
-	tprintf(", %d", (int) tcp->u_arg[4]);
+	tprint_arg_next();
+
+	/* optlen */
+	PRINT_VAL_D((int) tcp->u_arg[4]);
 
 	return RVAL_DECODED;
 }
